/*
    futebus - FUse TExt BUS - fuse based IPC bus for text lines
    Copyright (C) 2023  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://www.repo.hu/projects/futebus
    VCS: svn://svn.repo.hu/futebus/trunk
    Contact: see doc/contact.html
*/

#include <errno.h>

#include "futebus.h"
#include "channel.h"
#include "client.h"
#include "stat.h"

channel_t *statch = NULL;

#define APPENDS(str)  gds_append_str(&client->rbuf, str)
#define APPENDC(chr)  gds_append(&client->rbuf, chr)

static void stat_gen_chanlist(client_t *client, gdl_list_t *lst)
{
	channel_t *ch;
	client_t *cl;
	char tmp[256];

	for(ch = gdl_first(lst); ch != NULL; ch = gdl_next(lst, ch)) {
		mutex_lock(&ch->lock);
		sprintf(tmp, " channel #%ld (", (long)ch->id);
		APPENDS(tmp);
		APPENDS(ch->path);
		sprintf(tmp, " with %ld messages)\n", (long)ch->messages.length);
		APPENDS(tmp);


		for(cl = gdl_first(&ch->clients); cl != NULL; cl = gdl_next(&ch->clients, cl)) {
			sprintf(tmp, "  client #%ld", (long)cl->id);
			APPENDS(tmp);
			if (cl->master)
				APPENDS(" MASTER");
			if (cl->wr)
				APPENDS(" wr");
			if (cl->rd) {
				sprintf(tmp, " rd: rlist=%ld rbuf=%ld\n", (long)cl->rlist.used, (long)cl->rbuf.used);
				APPENDS(tmp);
			}
			else
				APPENDC('\n');
			
		}
		mutex_unlock(&ch->lock);
	}
	APPENDC('\n');
}

static void stat_gen(client_t *client)
{
	char tmp[256];

	APPENDS("Futebus:\n");
	APPENDS(" version: ");
	APPENDS(vers);
	APPENDS("\n\n");

	APPENDS("Bus:\n");
	sprintf(tmp, " msg_size_max: %ld\n", (long)msg_size_max);
	APPENDS(tmp);
	sprintf(tmp, " write_size_max: %ld\n", (long)write_size_max);
	APPENDS(tmp);
	sprintf(tmp, " rlist_size_max: %ld\n", rlist_size_max);
	APPENDS(tmp);

	APPENDS(" foreground: ");
	APPENDS(foreground ? "yes\n" : "no\n");

	APPENDS(" logging: ");
	if (logfile == stdout)
		APPENDS("to stdout\n");
	else if (logfile == NULL)
		APPENDS("no\n");
	else
		APPENDS("to file\n");
	APPENDC('\n');



	mutex_lock(&channels_lock);
	APPENDS("Active channels:\n");
	stat_gen_chanlist(client, &channels);

	APPENDS("Zombie channels:\n");
	stat_gen_chanlist(client, &zombie_channels);
	mutex_unlock(&channels_lock);

	client->rbuf_eof = 1;
}

int stat_open(const char *path, uint64_t *fh)
{
	client_t *client;

	if (statch == NULL) {
		statch = chan_alloc(path);
		if (statch == NULL)
			return -ENOMEM;
	}

	client = client_alloc_to_chan(statch, 1, 0, 0);
	if (client == NULL)
		return -ENOMEM;

	stat_gen(client);

	*fh = chan_client_to_fd(statch->id, client->id);
	return 0;
}

void stat_uninit(void)
{
	if (statch != NULL)
		chan_destroy_(statch);
}
